package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.deviceTenant.domain.SmDeviceTenant;
import com.ruoyi.ss.deviceTenant.service.ISmDeviceTenantService;
import com.ruoyi.ss.store.domain.StoreQuery;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.suit.domain.SuitQuery;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillGroupBy;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/4/13
 */
@Service
public class DeviceAssemblerImpl implements DeviceAssembler {

    @Autowired
    private ISmDeviceTenantService deviceTenantService;

    @Autowired
    private SuitService suitService;

    @Autowired
    private IStoreService storeService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private IotService iotService;

    @Override
    public void assembleTenant(List<DeviceVO> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        Map<Long, List<SmDeviceTenant>> tenantGroup = deviceTenantService.selectGroupByDeviceIds(list.stream().map(DeviceVO::getDeviceId).collect(Collectors.toList()));

        for (DeviceVO device : list) {
            List<SmDeviceTenant> tenants = tenantGroup.get(device.getDeviceId());
            if (CollectionUtils.isEmpty(tenants)) {
                continue;
            }
            device.setTenantIds(tenants.stream().map(SmDeviceTenant::getTenantId).collect(Collectors.toList()));
        }
    }

    /**
     * 拼接套餐列表
     *
     * @param list
     */
    @Override
    public void assembleSuitList(List<DeviceVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        SuitQuery query = new SuitQuery();
        query.setDeviceIds(list.stream().map(DeviceVO::getDeviceId).collect(Collectors.toList()));
        Map<Long, List<SuitVo>> groups = suitService.selectGroup(query, SuitVo::getDeviceId);

        for (DeviceVO device : list) {
            device.setSuitList(groups.get(device.getDeviceId()));
        }
    }

    /**
     * 拼接店铺营业时间
     *
     * @param list
     */
    @Override
    public void assembleBusinessTime(List<DeviceVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        StoreQuery query = new StoreQuery();
        query.setStoreIds(list.stream().map(DeviceVO::getStoreId).filter(Objects::nonNull).distinct().collect(Collectors.toList()));
        Map<Long, StoreVo> map = storeService.selectMap(query, StoreVo::getStoreId);

        for (DeviceVO device : list) {
            StoreVo store = device.getStoreId() == null ? null : map.get(device.getStoreId());
            if (store != null) {
                device.setStoreBusinessTimeStart(store.getBusinessTimeStart());
                device.setStoreBusinessTimeEnd(store.getBusinessTimeEnd());
            }
        }

    }

    /**
     * 拼接订单统计信息：订单数量、订单金额
     *
     * @param list
     */
    @Override
    public void assembleOrderCountInfo(List<DeviceVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        TransactionBillQuery query = new TransactionBillQuery();
        query.setGroupBy(TransactionBillGroupBy.device_id.name());
        query.setDeviceIds(list.stream().map(DeviceVO::getDeviceId).collect(Collectors.toList()));
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        Map<Long, BillCountVo> map = transactionBillService.selectCountMap(query, BillCountVo::getDeviceId);

        for (DeviceVO device : list) {
            BillCountVo count = map.get(device.getDeviceId());
            device.setOrderCount(count == null ? 0 : count.getCount());
            device.setOrderAmount(count == null ? BigDecimal.ZERO : count.getRecharge());
        }
    }

    @Override
    public void assembleIotDeviceInfo(List<DeviceVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 获取当前设备数据
        List<String> deviceNames = list.stream().map(DeviceVO::getMac).filter(StringUtils::hasText).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(deviceNames)) {
            return;
        }
        Map<String, IotDeviceInfo> deviceInfoMap = iotService.getDeviceInfo(deviceNames)
                .stream().collect(Collectors.toMap(IotDeviceInfo::getMac, item -> item));

        // 拼接数据
        for (DeviceVO device : list) {
            if (StringUtils.hasText(device.getMac())) {
                // 在线状态
                device.setOnlineStatus(iotService.getOnlineStatus(device.getMac()).getStatus());

                // 其他信息
                IotDeviceInfo deviceInfo = deviceInfoMap.get(device.getMac());
                if (deviceInfo != null) {
                    device.setElectricity(deviceInfo.getA());
                    device.setVoltage(deviceInfo.getV());
                    device.setRealTimePower(deviceInfo.getP());
                    device.setTotalElectriQuantity(deviceInfo.getW());  // 电量
                    device.setRemainTime(deviceInfo.getTime());
                }
            } else {
                device.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
            }
        }
    }
}
