package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.response.CommandResponse;
import com.ruoyi.iot.enums.IotHttpStatus;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.domain.DeviceCountVO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.dto.DeviceBatchUpdateModelDTO;
import com.ruoyi.ss.device.domain.dto.DeviceRegisterDTO;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.mapper.DeviceMapper;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.deviceBindRecord.service.ISmDeviceBindRecordService;
import com.ruoyi.ss.model.domain.SmModelVO;
import com.ruoyi.ss.model.service.ModelService;
import com.ruoyi.ss.record.time.service.IRecordTimeService;
import com.ruoyi.ss.record.time.service.RecordTimeConverter;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.timeBill.domain.TimeBillQuery;
import com.ruoyi.ss.timeBill.domain.TimeBillVO;
import com.ruoyi.ss.timeBill.service.TimeBillService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
@Slf4j
public class DeviceServiceImpl implements DeviceService
{
    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IotService iotService;

    @Autowired
    private ISmDeviceBindRecordService smDeviceBindRecordService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IStoreService storeService;

    @Autowired
    private IRecordTimeService recordTimeService;

    @Autowired
    private RecordTimeConverter recordTimeConverter;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private SuitService suitService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ModelService modelService;

    @Autowired
    private TimeBillService timeBillService;

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public DeviceVO selectSmDeviceByDeviceId(Long deviceId)
    {
        return deviceMapper.selectSmDeviceByDeviceId(deviceId);
    }

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备
     */
    @Override
    public List<DeviceVO> selectSmDeviceList(DeviceQuery smDevice)
    {
        return deviceMapper.selectSmDeviceList(smDevice);
    }

    /**
     * 新增设备
     * @param data 设备
     * @return 结果
     */
    @Override
    public int insertSmDevice(Device data) {
        ServiceUtil.assertion(data == null, "参数错误，不允许为空");
        ServiceUtil.assertion(StringUtils.isBlank(data.getDeviceNo()), "参数错误，SN不允许为空");
        ServiceUtil.assertion(StringUtils.isBlank(data.getMac()), "参数错误，MAC不允许为空");

        String key = data.getDeviceNo();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.ADD_DEVICE, key), "当前录入人数过多，请稍后再试");
        try {
            ServiceUtil.assertion(deviceValidator.isRepeatMac(data.getDeviceId(), data.getMac()), "MAC重复");
            ServiceUtil.assertion(deviceValidator.isRepeatSn(data.getDeviceId(), data.getDeviceNo()), "SN重复");

            // 创建OneNet设备
            int code = iotService.create(data.getMac());
            ServiceUtil.assertion(!IotHttpStatus.SUCCESS.equalCode(code) && !IotHttpStatus.DEVICE_EXIST.equalCode(code), "设备注册失败");

            data.setCreateTime(DateUtils.getNowDate());
            data.setStatus(DeviceStatus.NORMAL.getStatus());

            Integer result = transactionTemplate.execute(status -> {
                int insert = deviceMapper.insertSmDevice(data);
                ServiceUtil.assertion(insert != 1, "新增设备失败");

                DeviceQuery snQuery = new DeviceQuery();
                snQuery.setDeviceNo(data.getDeviceNo());
                int snCount = selectCount(snQuery);
                ServiceUtil.assertion(snCount != 1, "SN重复");

                DeviceQuery macQuery = new DeviceQuery();
                macQuery.setMac(data.getMac());
                int macCount = selectCount(macQuery);
                ServiceUtil.assertion(macCount != 1, "MAC重复");

                return insert;
            });
            return result == null ? 0 : result;
        } finally {
            redisLock.unlock(RedisLockKey.ADD_DEVICE, key);
        }
    }

    /**
     * 修改设备
     *
     * @param data 设备
     * @return 结果
     */
    @Override
    @Transactional
    public int updateSmDevice(Device data)
    {
        if (StringUtils.hasText(data.getDeviceNo())) {
            ServiceUtil.assertion(deviceValidator.isRepeatSn(data.getDeviceId(), data.getDeviceNo()), "SN重复");
        }

        // 修改数据库
        data.setUpdateTime(DateUtils.getNowDate());
        int updateCount = deviceMapper.updateSmDevice(data);

        return updateCount;
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds)
    {
        this.validatePreDelete(Arrays.asList(deviceIds));
        return deviceMapper.deleteSmDeviceByDeviceIds(deviceIds);
    }

    /**
     * 真正的删除校验
     */
    private void validatePreDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(true);
        Map<Long, DeviceVO> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            DeviceVO device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }
    }

    /**
     * 逻辑删除校验
     */
    private void validatePreLogicDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");

        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(false);
        // 前台用户，判断是不是他的设备
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            dto.setUserId(SecurityUtils.getUserId());
        }
        Map<Long, DeviceVO> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            DeviceVO device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
            ServiceUtil.assertion(DeviceStatus.USING.getStatus().equals(device.getStatus()), String.format("设备“%s”正在使用中，无法删除", deviceId));
        }

    }

    /**
     * 条件查询列表，并转为Map
     */
    private Map<Long, DeviceVO> selectMap(DeviceQuery dto) {
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(DeviceVO::getDeviceId, item -> item));
    }

    @Override
    public int selectCount(DeviceQuery dto) {
        return deviceMapper.selectCount(dto);
    }

    @Override
    public List<DeviceVO> selectSmDeviceByStoreId(Long storeId) {
        DeviceQuery dto = new DeviceQuery();
        dto.setStoreId(storeId);
        dto.setDeleted(false);
        return deviceMapper.selectSmDeviceList(dto);
    }


    @Override
    public boolean addTimeByUser(Long deviceId, long seconds, boolean withIot, String reason) {
        boolean addTime = this.addTime(deviceId, seconds, withIot);

        if (addTime) {
            // 记录下操作日志
            LoginUser loginUser = SecurityUtils.getLoginUser();
            scheduledExecutorService.schedule(() -> {
                DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
                recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, seconds, reason, loginUser));
            }, 0, TimeUnit.SECONDS);
        }

        return addTime;
    }

    @Override
    public int register(DeviceRegisterDTO dto) {
        if (dto == null) {
            return 0;
        }
        SmModelVO model = modelService.selectSmModelByModelId(dto.getModelId());
        ServiceUtil.assertion(model == null, "型号不存在");

        // 添加
        Device device = new Device();
        device.setDeviceName("未命名");
        device.setMac(dto.getMac());
        device.setDeviceNo(dto.getSn());
        device.setModelId(dto.getModelId());
        device.setServiceType(model.getServiceType());
        device.setServiceRate(model.getServiceRate());
        return this.insertSmDevice(device);
    }

    @Override
    public List<String> selectAllMac() {
        return deviceMapper.selectAllMac();
    }

    @Override
    public DeviceVO selectOne(DeviceQuery query) {
        return deviceMapper.selectOne(query);
    }

    @Override
    public List<DeviceVO> selectUsingDevice(Long userId) {
        // 查询正在使用中的订单
        List<TransactionBillVO> usingBills = transactionBillService.selectUsingBill(userId);
        List<Long> deviceIds = usingBills.stream().map(TransactionBillVO::getDeviceId).distinct().collect(Collectors.toList());
        // 查询设备列表
        return this.selectListByIds(deviceIds);
    }

    @Override
    public List<Long> selectDeviceUsingUser(Long deviceId) {
        // 正在使用的用户ID列表
        Set<Long> userIds = new HashSet<>();

        // 充值订单
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        List<TransactionBillVO> transactionList = transactionBillService.selectUsingBill(query);
        if (CollectionUtils.isNotEmptyElement(transactionList)) {
            userIds.addAll(transactionList.stream().map(TransactionBillVO::getUserId).collect(Collectors.toList()));
        }

        // 时长订单
        TimeBillQuery timeQuery = new TimeBillQuery();
        timeQuery.setDeviceId(deviceId);
        List<TimeBillVO> timeBillList = timeBillService.selectUsingList(timeQuery);
        if (CollectionUtils.isNotEmptyElement(timeBillList)) {
            userIds.addAll(timeBillList.stream().map(TimeBillVO::getUserId).collect(Collectors.toList()));
        }

        return new ArrayList<>(userIds);
    }

    /**
     * 开关设备
     */
    @Override
    public boolean switchDevice(Long deviceId, boolean open) {
        DeviceVO device = selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        if (open) {
            return iotService.open(device.getMac());
        } else {
            return iotService.close(device.getMac());
        }
    }

    @Override
    public int batchUpdateModel(DeviceBatchUpdateModelDTO dto) {
        return deviceMapper.batchUpdateModel(dto);
    }

    @Override
    public int updateServiceRate(DeviceVO data) {
        return deviceMapper.updateServiceRate(data);
    }


    @Override
    public boolean addTime(Long deviceId, long seconds, boolean withIot) {
        ServiceUtil.assertion( seconds < 0, "增加的时长不允许小于0");

        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备MAC号为空");
        ServiceUtil.assertion(DeviceStatus.FIXING.getStatus().equals(device.getStatus()), "设备正在维修中，无法使用");

        transactionTemplate.execute(status -> {
            // 更新数据库时长
            int updateCount = deviceMapper.addTime(deviceId, seconds);
            ServiceUtil.assertion(updateCount != 1, "增加时长失败，请刷新后重试");

            // 修改状态为使用中
            changeStatus(deviceId, DeviceStatus.USING);

            // 物联网设备增加时长
            if (withIot) {
                DeviceVO newDevice = selectSmDeviceByDeviceId(deviceId);
                long betweenSeconds = Duration.between(LocalDateTime.now(), newDevice.getExpireTime()).getSeconds();
                if (betweenSeconds > 0) {
                    CommandResponse rechargeResult = iotService.setTime(device.getMac(), betweenSeconds);
                    ServiceUtil.assertion(!rechargeResult.isSuccess(), "设备充值失败，请检查设备是否在线或联系管理员");
                }
            }
            return Boolean.TRUE;
        });


        // 拉取设备信息
        this.pullDeviceInfoAsync(Collections.singletonList(deviceId), 3, TimeUnit.SECONDS);

        // 时长结束后修改设备状态
        scheduledExecutorService.schedule(()-> {
            freshStatus(deviceId);
        }, seconds, TimeUnit.SECONDS);

        return true;
    }


    private void freshStatus(Long deviceId) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        freshStatus(device);
    }

    @Override
    public void freshStatus(DeviceVO device) {
        if (device == null) {
            return;
        }
        if (!DeviceStatus.USING.getStatus().equals(device.getStatus())) {
            return;
        }
        LocalDateTime now = LocalDateTime.now();
        Duration between = Duration.between(now, device.getExpireTime());

        // 如果已过期，则调整设备为正常状态
        int i = deviceMapper.changeTimeoutStatus(device.getDeviceId());
        if (i == 1) {
            return;
        }

        // 如果没有过期，则延迟至过期时间继续查询
        scheduledExecutorService.schedule(() -> {
            freshStatus(device.getDeviceId());
        }, between.getSeconds() , TimeUnit.SECONDS);
    }

    /**
     * 设备心跳，用于检测是否在线
     */
    @Override
    public void deviceHeartBeat() {
        log.info("device heart beat running...");
        List<DeviceVO> list = deviceMapper.selectSimpleList(new DeviceQuery());
        if (CollectionUtils.isEmpty(list)) {
            log.info("device list is empty");
            return;
        }
        for (DeviceVO device : list) {
            if (StringUtils.hasText(device.getMac())) {
                String status = iotService.getOnlineStatus(device.getMac()).getStatus();
                log.info("device: {} {} online status is {}", device.getDeviceId(), device.getMac(), status);
                device.setOnlineStatus(status);
            } else {
                device.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
                log.info("device {} mac is empty", device.getDeviceId());
            }
        }
        int i = deviceMapper.batchUpdateOnlineStatus(list);
        log.info("update count: {}", i);
    }


    private int changeStatus(Long deviceId, DeviceStatus status) {
        Device device = new Device();
        device.setDeviceId(deviceId);
        device.setStatus(status.getStatus());
        return deviceMapper.updateSmDevice(device);
    }

    @Override
    public boolean logicDel(List<Long> deviceIds) {
        this.validatePreLogicDelete(deviceIds);
        return deviceMapper.logicDel(deviceIds) > 0;
    }

    @Override
    public Map<Long, List<DeviceVO>> selectGroupByModelIds(List<Long> modelIds) {
        if (CollectionUtils.isEmpty(modelIds)) {
            return Collections.emptyMap();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setDeleted(false);
        dto.setModelIds(modelIds);
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(DeviceVO::getModelId));
    }

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    @Override
    public boolean changePowerStatus(Long deviceId, DevicePowerStatus status) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备Mac号为空");

        deviceAssembler.assembleTenant(Collections.singletonList(device));
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的商户，无法进行该操作" );

        if (DevicePowerStatus.ON.equals(status)) {
            iotService.open(device.getMac());
        } else if (DevicePowerStatus.OFF.equals(status)) {
            iotService.close(device.getMac());
        } else {
            throw new ServiceException("不支持的操作");
        }

        // 修改数据库通断电状态
        Device changePowerForm = new Device();
        changePowerForm.setDeviceId(deviceId);
        changePowerForm.setPowerStatus(status.getStatus());
        deviceMapper.updateSmDevice(changePowerForm);

        return true;
    }

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    @Override
    @SneakyThrows
    public void pullDeviceInfo(List<Long> deviceIds) {
        // 查询设备列表
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        List<DeviceVO> list = deviceMapper.selectSimpleList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        // 获取数据点信息
        List<String> deviceNames = list.stream().map(DeviceVO::getMac).filter(StringUtils::hasText).collect(Collectors.toList());
        List<IotDeviceInfo> deviceInfos = iotService.getDeviceInfo(deviceNames);

        // 更新数据
        for (DeviceVO device : list) {
            try {
                IotDeviceInfo deviceInfo = deviceInfos.stream().filter(item -> Objects.equals(device.getMac(), item.getMac())).findFirst().orElse(null);
                Device data = new Device();
                data.setDeviceId(device.getDeviceId());

                // 设备信息
                if (deviceInfo != null) {
                    data.setLastPullTime(deviceInfo.getAt());
                    data.setTotalElectriQuantity(deviceInfo.getW());
                    data.setPowerStatus(deviceInfo.getS());
                    data.setRemainTime(deviceInfo.getTime());
                    data.setWifi(deviceInfo.getWifi());
                }

                // 是否在线
                data.setOnlineStatus(iotService.getOnlineStatus(device.getMac()).getStatus());

                deviceMapper.updateSmDevice(data);
            } catch (Exception e) {
                log.error("更新设备信息失败，deviceId:{}", device.getDeviceId(), e);
            }
        }
    }

    /**
     * 异步延迟执行拉取设备信息
     * @param deviceIds 设备id列表
     * @param delay 延迟时间
     * @param unit 时间单位
     */
    private void pullDeviceInfoAsync(List<Long> deviceIds, long delay, TimeUnit unit) {
        scheduledExecutorService.schedule(() -> {
            log.info("异步拉取设备信息 start");
            this.pullDeviceInfo(deviceIds);
            log.info("异步拉取设备信息 end");
        }, delay, unit);
    }



    @Override
    public String bindBySn(Long storeId, Long userId, String deviceNo) {
        DeviceVO device = selectByDeviceNo(deviceNo);
        return this.bind(storeId, userId, device);
    }

    @Override
    public String bindByMac(Long storeId, Long userId, String mac) {
        DeviceVO device = selectByMac(mac);
        return this.bind(storeId, userId, device);
    }

    private String bind(Long storeId, Long userId, DeviceVO device) {
        ServiceUtil.assertion(device == null, "设备未录入");
        // 如果绑定的用户是原来的用户，则直接返回MAC
        if (device.getUserId() != null) {
            if (Objects.equals(device.getUserId(), userId)) {
                return device.getMac();
            } else {
                throw new ServiceException("该设备已被绑定");
            }
        }
        // 如果绑定的是原来的店铺，则直接返回MAC
        if (device.getStoreId() != null) {
            if (Objects.equals(device.getStoreId(), storeId)) {
                return device.getMac();
            } else {
                throw new ServiceException("该设备已被绑定");
            }
        }

        // 查询用户
        SmUserVo user = userService.selectSimpleById(userId);
        ServiceUtil.assertion(user == null, "用户不存在");

        // 查询店铺
        if (storeId != null) {
            StoreVo store = storeService.selectSmStoreById(storeId);
            ServiceUtil.assertion(store == null, "店铺不存在");
            ServiceUtil.assertion(!Objects.equals(store.getUserId(), userId), "该店铺不属于该用户");
        }

        // 绑定
        int updateCount = deviceMapper.bindStore(device.getDeviceId(), storeId, userId);
        ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

        // 设备绑定记录
        scheduledExecutorService.schedule(() -> {
            smDeviceBindRecordService.record(userId, device.getDeviceId());
        }, 0, TimeUnit.SECONDS);

        return device.getMac();
    }

    /**
     * 根据mac号查询数据
     * @param mac
     * @return
     */
    @Override
    public DeviceVO selectByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return null;
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setMac(mac);
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public boolean reset(Long deviceId) {
        ServiceUtil.assertion(deviceId == null, "设备id不能为空");

        LocalDateTime now = LocalDateTime.now();
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的商户，无法进行该操作");

        // 归零时间，将过期时间设置为现在
        Device form = new Device();
        form.setDeviceId(deviceId);
        form.setExpireTime(now);
        form.setStatus(DeviceStatus.NORMAL.getStatus());
        deviceMapper.updateSmDevice(form);

        // 关闭该设备未结束的所有订单
        transactionBillService.batchEndBillByDevice(deviceId);

        // 物联网设备归零
        CommandResponse commandResponse = iotService.setTime(device.getMac(), 0L);
        ServiceUtil.assertion(!commandResponse.isSuccess(), "设备归零失败，请检查设备是否在线或联系管理员");

        // 强制关闭
        iotService.close(device.getMac());

        // 归零记录
        LoginUser loginUser = SecurityUtils.getLoginUser();
        scheduledExecutorService.schedule(() -> {
            // 设备剩余时长
            Duration duration = Duration.between(now, device.getExpireTime());
            recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, -duration.getSeconds(), "设备归零", loginUser));
        }, 0, TimeUnit.SECONDS);

        return true;
    }

    /**
     * 设备是否已经被绑定
     *
     * @param mac
     * @return
     */
    @Override
    public int bindCodeByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return Constants.DEVICE_NOT_FOUND;
        }
        DeviceVO device = deviceMapper.selectSimpleSmDeviceByMac(mac);
        return isBindCode(device);
    }

    private int isBindCode(DeviceVO device) {
        if (device == null) {
            return Constants.DEVICE_NOT_FOUND;
        }
        if (device.getUserId() == null) {
            return Constants.DEVICE_NOT_BIND;
        } else {
            return Constants.DEVICE_BINDED;
        }
    }

    /**
     * 根据SN码查询设备是否已经绑定
     *
     * @param deviceNo
     * @return
     */
    @Override
    public int bindCodeBySn(String deviceNo) {
        if (StringUtils.isBlank(deviceNo)) {
            return Constants.DEVICE_NOT_FOUND;
        }
        DeviceVO device = deviceMapper.selectSimpleSmDeviceBySn(deviceNo);
        return isBindCode(device);
    }

    /**
     * 更新设备服务费
     *
     * @param userId
     * @param serviceRate
     */
    @Override
    public int updateServiceRateByUserId(Long userId, BigDecimal serviceRate) {
        if (userId == null) {
            return 0;
        }
        List<StoreVo> list = storeService.selectByUserId(userId);
        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }
        return deviceMapper.updateServiceRateByStores(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()), serviceRate);
    }

    @Override
    public int updateByIot(Device device) {
        device.setLastPullTime(DateUtils.getNowDate());
        return deviceMapper.updateSmDeviceByIot(device);
    }

    /**
     * ids查询列表
     *
     * @param ids
     */
    @Override
    public List<DeviceVO> selectListByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(ids);
        return selectSmDeviceList(dto);
    }

    /**
     * 查询各店铺的设备数量,并店铺
     *
     * @param storeIds 店铺id
     */
    @Override
    public Map<Long, Integer> selectCountMapByStoreIds(List<Long> storeIds) {
        if (CollectionUtils.isEmptyElement(storeIds)) {
            return Collections.emptyMap();
        }
        List<DeviceCountVO> list = deviceMapper.selectCountByStoreIds(storeIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(DeviceCountVO::getStoreId, DeviceCountVO::getCount));
    }

    /**
     * 条件查询数量，并店铺
     *
     * @param query 查询条件
     * @param keyMapper 键值的映射
     */
    @Override
    public <K> Map<K, Integer> selectCommonCountMap(DeviceQuery query, Function<? super DeviceCountVO, ? extends K> keyMapper ) {
        if (query == null) {
            return Collections.emptyMap();
        }
        List<DeviceCountVO> list = deviceMapper.selectCommonCount(query);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, DeviceCountVO::getCount));
    }

    /**
     * 获取设备，包含套餐列表
     *
     * @param deviceSn
     * @return
     */
    @Override
    public DeviceVO getDeviceWithSuitList(String deviceSn) {
        if (deviceSn == null) {
            return null;
        }
        DeviceVO device = deviceMapper.selectByDeviceNo(deviceSn);
        if (device != null) {
            deviceAssembler.assembleSuitList(Collections.singletonList(device));
        }
        return device;
    }

    /**
     * 根据设备编号查询
     *
     * @param deviceNo
     */
    @Override
    public DeviceVO selectByDeviceNo(String deviceNo) {
        return deviceMapper.selectByDeviceNo(deviceNo);
    }

    @Override
    @Transactional
    public int unbind(Long deviceId) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(DeviceStatus.USING.getStatus().equals(device.getStatus()), "设备正在使用中，无法解除绑定");

        Integer result = transactionTemplate.execute(status -> {
            int updateCount = deviceMapper.unbindStore(deviceId);
            ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

            // 删除套餐
            suitService.logicDelByDeviceId(deviceId);

            return updateCount;
        });

        return result == null ? 0 : result;
    }

    /**
     * 设备绑定SN码
     *
     * @param deviceId 设备ID
     * @param sn       SN
     */
    @Override
    public int bindSn(Long deviceId, String sn) {
        Integer result = transactionTemplate.execute(status -> {
            ServiceUtil.assertion(deviceValidator.isRepeatSn(deviceId, sn), "SN重复");

            int i = deviceMapper.bindSn(deviceId, sn);
            ServiceUtil.assertion(i != 1, "设备SN已发生变化，请刷新后重试");
            return i;
        });
        return result == null ? 0 : result;
    }
}
