package com.ruoyi.ss.refund.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.ss.refund.domain.RefundQuery;
import com.ruoyi.ss.refund.domain.RefundVO;
import com.ruoyi.ss.refund.domain.enums.RefundStatus;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.service.ISmUserService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.refund.mapper.RefundMapper;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.service.RefundService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 退款订单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-07-09
 */
@Service
public class RefundServiceImpl implements RefundService
{
    @Autowired
    private RefundMapper refundMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private ISmUserService userService;

    /**
     * 查询退款订单
     *
     * @param refundId 退款订单主键
     * @return 退款订单
     */
    @Override
    public RefundVO selectRefundByRefundId(Long refundId)
    {
        return refundMapper.selectRefundByRefundId(refundId);
    }

    /**
     * 查询退款订单列表
     *
     * @param refund 退款订单
     * @return 退款订单
     */
    @Override
    public List<RefundVO> selectRefundList(RefundQuery refund)
    {
        return refundMapper.selectRefundList(refund);
    }

    /**
     * 新增退款订单
     *
     * @param refund 退款订单
     * @return 结果
     */
    @Override
    public int insertRefund(Refund refund)
    {
        refund.setCreateTime(DateUtils.getNowDate());
        refund.setRefundNo(String.valueOf(SnowFlakeUtil.newId()));
        return refundMapper.insertRefund(refund);
    }

    /**
     * 修改退款订单
     *
     * @param refund 退款订单
     * @return 结果
     */
    @Override
    public int updateRefund(Refund refund)
    {
        return refundMapper.updateRefund(refund);
    }

    /**
     * 批量删除退款订单
     *
     * @param refundIds 需要删除的退款订单主键
     * @return 结果
     */
    @Override
    public int deleteRefundByRefundIds(Long[] refundIds)
    {
        return refundMapper.deleteRefundByRefundIds(refundIds);
    }

    /**
     * 删除退款订单信息
     *
     * @param refundId 退款订单主键
     * @return 结果
     */
    @Override
    public int deleteRefundByRefundId(Long refundId)
    {
        return refundMapper.deleteRefundByRefundId(refundId);
    }

    @Override
    public RefundVO selectRefundByRefundNo(String refundNo) {
        RefundQuery query = new RefundQuery();
        query.setRefundNo(refundNo);
        return this.selectOne(query);
    }

    @Override
    public RefundVO selectOne(RefundQuery query) {
        return refundMapper.selectOne(query);
    }

    @Override
    public void handleRefundSuccess(String refundNo) {
        ServiceUtil.assertion(StringUtils.isBlank(refundNo), "退款订单编号不允许为空");

        RefundVO refund = this.selectRefundByRefundNo(refundNo);
        ServiceUtil.assertion(refund == null, "退款订单不存在");

        TransactionBillVO bill = transactionBillService.selectSmTransactionBillByBillId(refund.getBillId());
        ServiceUtil.assertion(bill == null, "原订单不存在");

        transactionTemplate.execute(status -> {
            // 退款订单修改状态
            Refund refundData = new Refund();
            refundData.setStatus(RefundStatus.REFUND_SUCCESS.getStatus());
            RefundQuery refundQuery = new RefundQuery();
            refundQuery.setRefundId(refund.getRefundId());
            refundQuery.setStatus(RefundStatus.REFUNDING.getStatus());
            int updateRefund = this.updateByQuery(refundData, refundQuery);
            ServiceUtil.assertion(updateRefund != 1, "修改退款订单状态失败");

            // 修改原订单状态
            TransactionBillQuery billQuery = new TransactionBillQuery();
            billQuery.setBillId(refund.getBillId());
            billQuery.setStatus(TransactionBillStatus.REFUNDING.getStatus());
            TransactionBill billData = new TransactionBill();
            billData.setStatus(TransactionBillStatus.REFUNDED.getStatus());
            int updateBill = transactionBillService.updateByQuery(billData, billQuery);
            ServiceUtil.assertion(updateBill != 1, "修改原订单状态失败");

            return updateRefund;
        });

    }

    @Override
    public int updateByQuery(Refund data, RefundQuery query) {
        if (query == null) {
            return 0;
        }
        return refundMapper.updateByQuery(data, query);
    }
}
